// Copyright (C) 1998 Logi Ragnarsson

package is.hi.logir.cryptonite;

/**
 * This interface is implemented by classes to hold one cipher key.
 * It's descendants handle the actual encryption and decryption of
 * blocks of data.
 *
 * @author <a href="http://www.hi.is/~logir/">Logi Ragnarsson</a> (<a href="mailto:logir@hi.is">logir@hi.is</a>)
 */
public interface CipherKey extends Key {

    /**
     * Returns the size of the blocks that can be encrypted in one call
     * to encrypt(). */
    public int plainBlockSize();
    
    /**
     * Returns the size of the blocks that can be decryptes in one call
     * to decrypt(). */
    public int cipherBlockSize();
    
    /**
     * Encrypt one block of data. The plain data is taken from
     * <code>source</code> starting at offset <code>i</code> and
     * ciphertext is written to <code>dest</code>, starting at
     * offset <code>j</code>.
     * <p>
     * The amount of data read and written will match the values returned
     * by <code>plainBlockSize()</code> and <code>cipherBlockSize()</code>. */
    public void encrypt(byte[] source, int i, byte[] dest, int j);
    
    /**
     * Decrypt one block of data. The encrypted data is taken from
     * <code>source</code> starting at offset <code>i</code> and
     * plaintext is written to <code>dest</code>, starting at
     * offset <code>j</code>.
     * <p>
     * The amount of data read and written will match the values returned
     * by <code>cipherBlockSize()</code> and <code>plainBlockSize()</code>. */
    public void decrypt(byte[] source, int i, byte[] dest, int j);
    
}
