// Copyright (C) 1998 Logi Ragnarsson

package is.hi.logir.cryptonite;

import java.io.*;

/**
 * This class can be used to apply an interactive key exchange protocol to a
 * pair f streams and then encrypt all data going through them with the session
 * key exchanged. This class expects to talk to an equivalent client class.
 *
 * @see is.hi.logir.cryptonite.CipherStreamServer
 * @see is.hi.logir.cryptonite.InterKeyExServer
 *
 * @author <a href="http://www.hi.is/~logir/">Logi Ragnarsson</a>
 * (<a href="mailto:logir@hi.is">logir@hi.is</a>)
 */
public class CipherStreamClient extends CipherStream {

  /**
   * Create a new object which uses <code>kex</code> to exchange keys with
   * a remote server and then <code>encrypt</code> and <code>decrypt</code> to
   * encrypt and decrypt the data to/from <code>in</code> and <code>out</code>. 
   *
   * @exception ProtocolException if there is a problem exchanging keys.
   * @exception IOException if there is a problem with the underlying streams. */
  public CipherStreamClient(InputStream in, OutputStream out, InterKeyExClient kex, EncryptMode encrypt, DecryptMode decrypt) throws ProtocolException, IOException {
    if(kex!=null){
      // We are using key-exchange
      byte[] msg=kex.message(null);
      while(true){
	if(msg==null)
	  break;
        writeInt(out,msg.length);
        out.write(msg);
        if(kex.sessionKey()!=null)
          break;
	int l=readInt(in);
	if(msg.length!=l)
	  msg=new byte[l];
	in.read(msg);
	msg=kex.message(msg);
      }
      CipherKey key;
      try{
	key=(CipherKey)kex.sessionKey();
      } catch (ClassCastException e){
	throw new ProtocolException("The exchagned Key was not a CipherKey.");
      }
      encrypt.setKey(key);
      decrypt.setKey(key);
    }
    cOut = new EncryptStream(out, null, encrypt);
    cIn = new DecryptStream(in, null, decrypt);
    this.encrypt=encrypt;
    this.decrypt=decrypt;
  }

}