// Copyright (C) 1998 Logi Ragnarsson

package is.hi.logir.cryptonite;

import java.math.BigInteger;

/**
 * Diffie-Hellman key exchange client without exchanging keys. Both parties
 * need to know the other party's public DHKey. Nomessages are
 * sent in either direction, but a unique session key is created for each
 * pair of Diffie-Hellman keys used.
 *
 * @see is.hi.logir.cryptonite.DHKeyExServer
 * @see is.hi.logir.cryptonite.DHKeyExClient
 * @see is.hi.logir.cryptonite.DHKey
 *
 * @author <a href="http://www.hi.is/~logir/">Logi Ragnarsson</a>
 * (<a href="mailto:logir@hi.is">logir@hi.is</a>)
 */
public class DHKeyExNoninter extends DHKeyEx implements NoninterKeyExClient,NoninterKeyExServer {
    
    /**
     * Create a new  DHKeyExClient object which uses
     * the private DH key from <code>pri</code> and the public
     * DH key from <code>pri</code> to generate a session key.
     *
     * @exception KeyException if the public/private flag of either key
     *            is wrong.
     * @exception InvalidCDSException if the session key object can not
     *            be created.
     */
    public DHKeyExNoninter(DHKey pri, DHKey pub, String keyType) throws KeyException, InvalidCDSException {
        super(pri,keyType);
        byte[] k=pub.getKey().modPow(pri.getKey(),m).toByteArray();
        sessionKey=makeSessionKey(keyType,k);
        keyDecided=true;
    }
    
    /**
     * Never called, since no messages are needed for this 'protocol'.
     *
     * @exception ProtocolException if called
     */
    public byte[] message(byte[] received) throws ProtocolException {
        throw new ProtocolException("No messages should be sent.");
    }

}