// Copyright (C) 1998 Logi Ragnarsson

package is.hi.logir.cryptonite;

/**
 * Exchange keys by sending an encrypted key from this class to the
 * corresponding EncryptedKeyExServer.
 *
 * @author <a href="http://www.hi.is/~logir/">Logi Ragnarsson</a>
 * (<a href="mailto:logir@hi.is">logir@hi.is</a>) */
public class EncryptedKeyExClient extends EncryptedKeyEx implements NoninterKeyExClient {
  
  /**
   */
  public EncryptedKeyExClient(CipherKey key, Key sessionKey){
    super(key,sessionKey);
  }
  
  /**
   * Return the message to send to the server.
   *
   * @exception ProtocolException if the message is not null
   */
  public byte[] message(byte[] received) throws ProtocolException {
    if(received!=null)
      throw new ProtocolException("A non-interactive key-exchange client should not receive messages.");

    byte[] plain=sessionKey.toString().getBytes();
    EncryptMode em=new EncryptECB(key);
    byte[] c1=em.encrypt(plain,0,plain.length);
    byte[] c2=em.flush();

    byte[] cipher=new byte[c1.length+c2.length+4];
    writeBytes(plain.length, cipher,0, 4);
    System.arraycopy(c1,0, cipher,4          , c1.length);
    System.arraycopy(c2,0, cipher,4+c1.length, c2.length);
    
    keyDecided=true;
    return cipher;
  }
  
}
