// Copyright (C) 1998 Logi Ragnarsson

package is.hi.logir.cryptonite;

/**
 * This class implements server portion of the key-exchange protocol
 * whereby the client sends the server the hash of the key that should
 * be used for decryption. The server is assumed to have this key in
 * its default KeySource.
 *
 * @see is.hi.logir.cryptonite.SendHashKeyExClient
 *
 * @author <a href="http://www.hi.is/~logir/">Logi Ragnarsson</a>
 * (<a href="mailto:logir@hi.is">logir@hi.is</a>) */
public class SendHashKeyExServer extends SendHashKeyEx implements NoninterKeyExServer {

  /** Create a new SendHashKeyExServer. */
  public SendHashKeyExServer(){
    super(null);
  }
  
  /**
   * Receive the message from the client.
   *
   * @exception ProtocolException if a malformed message is received or the
   *            key specified in the message is not found.
   */
  public byte[] message(byte[] received) throws ProtocolException {
    if(received==null)
      throw new ProtocolException("A KeyExServer should never send the first message.");
    try{
      KeyRecord kr = keySource.byFingerprint((Fingerprint)fromString(new String(received)));
      if (kr==null)
	throw new ProtocolException("No Key matching the specified Fingerprint found in keySource.");
      sessionKey = kr.getKey();
      keyDecided=true;
      return null;
    } catch (InvalidCDSException e1){
      throw new ProtocolException("A CDS for a Fingerprint object expected.");
    } catch (ClassCastException e2){
      throw new ProtocolException("A CDS for a Fingerprint object expected.");
    }
  }
  
}
