// Copyright (C) 1998 Logi Ragnarsson

package is.hi.logir.cryptonite;

/**
 * This class stores a signature.
 *
 * @see is.hi.logir.cryptonite.Fingerprint
 * @see is.hi.logir.cryptonite.SignatureKey
 *
 * @author <a href="http://www.hi.is/~logir/">Logi Ragnarsson</a>
 *         (<a href="mailto:logir@hi.is">logir@hi.is</a>)
 */
public class Signature extends Cryptonite {

  private byte[] s;             // encrypted fingerprint.
  private Fingerprint keyPrint; // Fingerprint of the key used to create the signature.
  private String fingAlg;       // Fingerptint algorithm used.
  
  /**
   * Create a new Signature object for the signature <code>s</code> which
   * was generated from a fingerprint using the specified
   * <code>algorithm</code>. It can be verified with the key with Fingerprint
   * <code>k</code>.  */
  public Signature(byte[] s, String algorithm, Fingerprint k){
    this.s = new byte[s.length];
    System.arraycopy(s,0, this.s,0, s.length);
    keyPrint = k;
  }
  
  /**
   * Return the name of the algorithm used to fingerprint the data
   * before signing. */
  public String getFingerprintAlgorithm(){
    return fingAlg;
  }
  
  /** Return the bytes from this signature. */
  public byte[] getBytes(){
    byte[] ss = new byte[s.length];
    System.arraycopy(s,0, ss,0, s.length);
    return ss;
  }
  
  /** Return the fingerprint of the key used to verify this signature. */
  public Fingerprint getKeyPrint(){
    return keyPrint;
  }
  
  /** Return a CDS for this object. */
  public String toString(){
    StringBuffer sb=new StringBuffer();
    sb.append("Signature(");
    sb.append(hexString(s));
    sb.append(',');
    sb.append(keyPrint.toString());
    sb.append(')');
    return sb.toString();
  }
  
}
