// Copyright (C) 1998 Logi Ragnarsson

package is.hi.logir.cryptonite;

/**
 * This interface should only be implemented by keys that can be used to
 * create and validate signatures on blocks of data.
 * 
 * @author <a href="http://www.hi.is/~logir/">Logi Ragnarsson</a>
 * (<a href="mailto:logir@hi.is">logir@hi.is</a>) */
public interface SignatureKey extends Key {

  /**
   * Returns the maximum size in bytes of the fingerprint
   * that can be signed. */
  public int signBlockSize();

  /**
   * Returns the length of the signature in bytes. */
  public int signatureSize();

  /**
   * Create a signature for the fingerprint <code>fp</code>.
   *
   * @exception KeyException if this key can't be used to sign
   *            this fingerprint for some reason. */
  public Signature sign(Fingerprint fp) throws KeyException ;
  
  /** 
   * Return true iff <code>s</code> is a signature for <code>fp</code>
   * created with this key in the case of a symmetric key, or the other
   * key in the pair in the case of an asymmetric key. */
  public boolean matches(Signature s, Fingerprint fp);
  
}
