// Copyright (C) 1998 Logi Ragnarsson

package is.hi.logir.cryptonite;

/**
 * This class is the ancestor of all symmetric keys classes.
 * <p>
 * 
 * @see is.hi.logir.cryptonite.KeyPair
 * @see is.hi.logir.cryptonite.KeyRing
 *
 * @author <a href="http://www.hi.is/~logir/">Logi Ragnarsson</a>
 * (<a href="mailto:logir@hi.is">logir@hi.is</a>)
 */
public abstract class SymmetricKey extends K {
    
    // -----------------------------------------------------------
    // INSTANCE METHODS
    
    /** 
     * Calculate the fingerprint for this key or the other in the pair. The
     * default behaviour is to return the fingerprint of the CDS for this
     * key. */
    protected Fingerprint calcFingerprint(boolean other){
        try {
            return fingerprint = Fingerprint.create(toString(),"SHA1");
        } catch (InvalidCDSException e) {
            throw new CryptoniteCorruptError("The SHA1State class is missing");
        }
    }
    
    /**
     * Returns true iff this is a private key.
     * <p>
     * Symmetric keys simply return <code>true</code>. */
    public boolean isPrivate(){
        return true;
    }
    

    /**
     * Returns true if this and key are a matched pair of public/private
     * keys or the same symmetric key.
     * <p>
     * Symmetric keys simply call <code>equals(key)</code>. */
    public boolean matches(Key key){
        return equals(key);
    }
    
}
