// Copyright (C) 1998 Logi Ragnarsson

package is.hi.logir.cryptonite.test;

import is.hi.logir.cryptonite.*;

import java.io.*;

/**
 * This application tests the various Key classes.
 *
 * @author <a href="http://www.hi.is/~logir/">Logi Ragnarsson</a>
 * (<a href="mailto:logir@hi.is">logir@hi.is</a>)
 *
 * @see is.hi.logir.cryptonite.Key
 */
public class TestSign extends Cryptonite{

  static void copyStream(InputStream in, OutputStream out) throws IOException {
    byte[] buf=new byte[1];
    int l;
    while((l=in.read(buf))!=-1){
      //System.err.println("["+l+"]");
      out.write(buf,0,l);
    }
  }
    
  static class VerifyThread extends Thread{

    InputStream in;
    
    public VerifyThread(InputStream in){
      this.in = in;
    }

    public void run(){
      try {
        copyStream(in,System.out);
      } catch (IOException e) {
        e.printStackTrace(System.err);
      }
    }
    
  }

  public static void main(String[] arg) throws Exception {
    System.out.println("Generating RSA Keys");
    KeyPair kp = RSAKey.createKeys(512);
    SignatureKey pub = (SignatureKey)kp.getPublic();
    SignatureKey pri = (SignatureKey)kp.getPrivate();

    PipedInputStream pin = new PipedInputStream();
    PipedOutputStream pout = new PipedOutputStream(pin);
    
    new VerifyThread(new VerifyStream(pin,
                                      512,
                                      pub,
                                      new SHA1State()
                                     )).start();
    
    OutputStream out = new SignStream(pout,
                                      512,
                                      pri,
                                      new SHA1State()
                                     );
    copyStream(System.in,out);
    out.close();
  }
  
}